#include <SoftwareSerial.h>

#define Rx 2
#define Tx 4

SoftwareSerial mySerial(Rx, Tx);
void findAverage(int stepTime);
void sendToBT(float x, float y);

//Initial setup for code.
void setup() {
  pinMode(3, OUTPUT);   //Set pin 3 to output voltage waveform.
  mySerial.begin(9600); //Initialize serial communication for board.
}

//Main body of code.
//Encompasses infinite loop of measurement.
void loop() {
  delay(20000); //Delay start of program. Allow for bluetooth connection.
  int maxVoltage = 150; //contains peak voltage in sweep.
  int voltageStep = 1;  //contains magnitude of each sweep step.
  int stepTime = 40;    //defines delay between each ping to ADC pins.
  while(1) {
    delay(10000); //Delay of 10s between each measurement. Can change or toggle.
    for(int i = 0; i < maxVoltage; i = i + voltageStep) {
      analogWrite(3, i);
      findAverage(stepTime);
    }
    for(int i = maxVoltage; i > 0; i = i - voltageStep) {
      analogWrite(3, i);
      findAverage(stepTime);
    }
  }
}

//Reads analog inputs through ADC.
//Sends measured values to the HC-05 chip.
void findAverage(int stepTime) {
  float voltage = 0.0;  //value of measure voltage.
  float adc0 = 0.0;     //value to store ADC A0 output.

  float current = 0.0;  //value of measured current.
  float adc2 = 0.0;     //value to store ADC A2 output.
  
  for(int i = 0; i < 8; i++) {
    //measure both current and voltage in tandem.
    delay(stepTime / 8);
    adc0 = analogRead(A0);
    adc2 = analogRead(A2);
    voltage = (voltage + adc0) / 2;
    current = (current + adc2) / 2;
  }
  //scale voltage and current to VOLTAGE VALUES.
  voltage = (voltage * 5) / 1023;
  //voltage = (voltage * -1) + 3.3;
  current = (current * 5) / 1023;
  //current = current - 3.3;
  sendToBT(voltage, current);
  return;
}

//sends inputs to the Tx pin for bluetooth transmission.
void sendToBT(float x, float y) {
  mySerial.println(x);
  //delay(25);
  //Serial.println(x);
  delay(25);
  mySerial.println(y);
  //delay(25);
  //Serial.println(y);
  return;
}